@@ -1,15 +1,9 @@
 # Last Change: Mon Aug 20 08:00 PM 2007 J
-from __future__ import division, print_function, absolute_import
-
 import re
-import itertools
 import datetime
-from functools import partial
-from collections import OrderedDict
 
 import numpy as np
 
-from scipy._lib.six import next
 import csv
 import ctypes
 
@@ -44,7 +38,7 @@ r_datameta = re.compile(r'^@[Dd][Aa][Tt][Aa]')
 r_relation = re.compile(r'^@[Rr][Ee][Ll][Aa][Tt][Ii][Oo][Nn]\s*(\S*)')
 r_attribute = re.compile(r'^\s*@[Aa][Tt][Tt][Rr][Ii][Bb][Uu][Tt][Ee]\s*(..*$)')
 
-r_nominal = re.compile('{(.+)}')
+r_nominal = re.compile(r'{(.+)}')
 r_date = re.compile(r"[Dd][Aa][Tt][Ee]\s+[\"']?(.+?)[\"']?$")
 
 # To get attributes name enclosed with ''
@@ -68,7 +62,7 @@ class ParseArffError(ArffError):
 # ----------
 # Attributes
 # ----------
-class Attribute(object):
+class Attribute:
 
     type_name = None
 
@@ -132,7 +126,8 @@ class NominalAttribute(Attribute):
         """
         m = r_nominal.match(atrv)
         if m:
-            return tuple(i.strip() for i in m.group(1).split(','))
+            attrs, _ = split_data_line(m.group(1))
+            return tuple(attrs)
         else:
             raise ValueError("This does not look like a nominal string")
 
@@ -155,7 +150,6 @@ class NominalAttribute(Attribute):
         """
         Parse a value of this type.
         """
-        data_str = data_str.strip()
         if data_str in self.values:
             return data_str
         elif data_str == '?':
@@ -338,7 +332,7 @@ class DateAttribute(Attribute):
                 "datetime64[%s]" % self.datetime_unit)
 
     def __str__(self):
-        return super(DateAttribute, self).__str__() + ',' + self.date_format
+        return super().__str__() + ',' + self.date_format
 
 
 class RelationalAttribute(Attribute):
@@ -385,7 +379,7 @@ class RelationalAttribute(Attribute):
                         [(a.name, a.dtype) for a in self.attributes])
 
     def __str__(self):
-        return (super(RelationalAttribute, self).__str__() + '\n\t' +
+        return (super().__str__() + '\n\t' +
                 '\n\t'.join(str(a) for a in self.attributes))
 
 
@@ -404,6 +398,72 @@ def to_attribute(name, attr_string):
     raise ParseArffError("unknown attribute %s" % attr_string)
 
 
+def csv_sniffer_has_bug_last_field():
+    """
+    Checks if the bug https://bugs.python.org/issue30157 is unpatched.
+    """
+
+    # We only compute this once.
+    has_bug = getattr(csv_sniffer_has_bug_last_field, "has_bug", None)
+
+    if has_bug is None:
+        dialect = csv.Sniffer().sniff("3, 'a'")
+        csv_sniffer_has_bug_last_field.has_bug = dialect.quotechar != "'"
+        has_bug = csv_sniffer_has_bug_last_field.has_bug
+
+    return has_bug
+
+
+def workaround_csv_sniffer_bug_last_field(sniff_line, dialect, delimiters):
+    """
+    Workaround for the bug https://bugs.python.org/issue30157 if is unpatched.
+    """
+    if csv_sniffer_has_bug_last_field():
+        # Reuses code from the csv module
+        right_regex = r'(?P<delim>[^\w\n"\'])(?P<space> ?)(?P<quote>["\']).*?(?P=quote)(?:$|\n)'
+
+        for restr in (r'(?P<delim>[^\w\n"\'])(?P<space> ?)(?P<quote>["\']).*?(?P=quote)(?P=delim)',  # ,".*?",
+                      r'(?:^|\n)(?P<quote>["\']).*?(?P=quote)(?P<delim>[^\w\n"\'])(?P<space> ?)',  # .*?",
+                      right_regex,  # ,".*?"
+                      r'(?:^|\n)(?P<quote>["\']).*?(?P=quote)(?:$|\n)'):  # ".*?" (no delim, no space)
+            regexp = re.compile(restr, re.DOTALL | re.MULTILINE)
+            matches = regexp.findall(sniff_line)
+            if matches:
+                break
+
+        # If it does not match the expression that was bugged, then this bug does not apply
+        if restr != right_regex:
+            return
+
+        groupindex = regexp.groupindex
+
+        # There is only one end of the string
+        assert len(matches) == 1
+        m = matches[0]
+
+        n = groupindex['quote'] - 1
+        quote = m[n]
+
+        n = groupindex['delim'] - 1
+        delim = m[n]
+
+        n = groupindex['space'] - 1
+        space = bool(m[n])
+
+        dq_regexp = re.compile(
+            r"((%(delim)s)|^)\W*%(quote)s[^%(delim)s\n]*%(quote)s[^%(delim)s\n]*%(quote)s\W*((%(delim)s)|$)" %
+            {'delim': re.escape(delim), 'quote': quote}, re.MULTILINE
+        )
+
+        doublequote = bool(dq_regexp.search(sniff_line))
+
+        dialect.quotechar = quote
+        if delim in delimiters:
+            dialect.delimiter = delim
+        dialect.doublequote = doublequote
+        dialect.skipinitialspace = space
+
+
 def split_data_line(line, dialect=None):
     delimiters = ",\t"
 
@@ -414,7 +474,10 @@ def split_data_line(line, dialect=None):
     # Remove the line end if any
     if line[-1] == '\n':
         line = line[:-1]
-
+    
+    # Remove potential trailing whitespace
+    line = line.strip()
+    
     sniff_line = line
 
     # Add a delimiter if none is present, so that the csv.Sniffer
@@ -424,6 +487,9 @@ def split_data_line(line, dialect=None):
 
     if dialect is None:
         dialect = csv.Sniffer().sniff(sniff_line, delimiters=delimiters)
+        workaround_csv_sniffer_bug_last_field(sniff_line=sniff_line,
+                                              dialect=dialect,
+                                              delimiters=delimiters)
 
     row = next(csv.reader([line], dialect))
 
@@ -434,7 +500,7 @@ def split_data_line(line, dialect=None):
 # Parsing header
 # --------------
 def tokenize_attribute(iterable, attribute):
-    """Parse a raw string in header (eg starts by @attribute).
+    """Parse a raw string in header (e.g., starts by @attribute).
 
     Given a raw string attribute, try to get the name and type of the
     attribute. Constraints:
@@ -509,8 +575,8 @@ def tokenize_single_comma(val):
         try:
             name = m.group(1).strip()
             type = m.group(2).strip()
-        except IndexError:
-            raise ValueError("Error while tokenizing attribute")
+        except IndexError as e:
+            raise ValueError("Error while tokenizing attribute") from e
     else:
         raise ValueError("Error while tokenizing single %s" % val)
     return name, type
@@ -524,8 +590,8 @@ def tokenize_single_wcomma(val):
         try:
             name = m.group(1).strip()
             type = m.group(2).strip()
-        except IndexError:
-            raise ValueError("Error while tokenizing attribute")
+        except IndexError as e:
+            raise ValueError("Error while tokenizing attribute") from e
     else:
         raise ValueError("Error while tokenizing single %s" % val)
     return name, type
@@ -584,7 +650,7 @@ def read_header(ofile):
     return relation, attributes
 
 
-class MetaData(object):
+class MetaData:
     """Small container to keep useful information on a ARFF dataset.
 
     Knows about attributes names and types.
@@ -609,15 +675,13 @@ class MetaData(object):
 
     Notes
     -----
-    Also maintains the list of attributes in order, i.e. doing for i in
+    Also maintains the list of attributes in order, i.e., doing for i in
     meta, where meta is an instance of MetaData, will return the
     different attribute names in the order they were defined.
     """
     def __init__(self, rel, attr):
         self.name = rel
-
-        # We need the dictionary to be ordered
-        self._attributes = OrderedDict((a.name, a) for a in attr)
+        self._attributes = {a.name: a for a in attr}
 
     def __repr__(self):
         msg = ""
@@ -634,7 +698,7 @@ class MetaData(object):
 
     def __getitem__(self, key):
         attr = self._attributes[key]
-        
+
         return (attr.type_name, attr.range)
 
     def names(self):
@@ -665,7 +729,7 @@ def loadarff(f):
     Read an arff file.
 
     The data is returned as a record array, which can be accessed much like
-    a dictionary of numpy arrays.  For example, if one of the attributes is
+    a dictionary of NumPy arrays. For example, if one of the attributes is
     called 'pressure', then its first 10 data points can be accessed from the
     ``data`` record array like so: ``data['pressure'][0:10]``
 
@@ -681,7 +745,7 @@ def loadarff(f):
        The data of the arff file, accessible by attribute names.
     meta : `MetaData`
        Contains information about the arff file such as name and
-       type of attributes, the relation (name of the dataset), etc...
+       type of attributes, the relation (name of the dataset), etc.
 
     Raises
     ------
@@ -699,8 +763,8 @@ def loadarff(f):
     * date type attributes
     * string type attributes
 
-    It can read files with numeric and nominal attributes.  It cannot read
-    files with sparse data ({} in the file).  However, this function can
+    It can read files with numeric and nominal attributes. It cannot read
+    files with sparse data ({} in the file). However, this function can
     read files with missing data (? in the file), representing the data
     points as NaNs.
 
@@ -747,7 +811,7 @@ def _loadarff(ofile):
         rel, attr = read_header(ofile)
     except ValueError as e:
         msg = "Error while parsing header, error was: " + str(e)
-        raise ParseArffError(msg)
+        raise ParseArffError(msg) from e
 
     # Check whether we have a string attribute (not supported yet)
     hasstr = False
@@ -773,7 +837,7 @@ def _loadarff(ofile):
     ni = len(attr)
 
     def generator(row_iter, delim=','):
-        # TODO: this is where we are spending times (~80%). I think things
+        # TODO: this is where we are spending time (~80%). I think things
         # could be made more efficiently:
         #   - We could for example "compile" the function, because some values
         #   do not change here.
